// build.sbt: A friendly build for an sbt project.
// Copyright (C) 2012-2014, 2017 Stephen Compall
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

import org.scalastyle.sbt.{ScalastylePlugin => SS}

name := "high-cost-of-anyval-subclasses"

version in ThisBuild := "0.1.0-SNAPSHOT"

organization in ThisBuild := "com.nocandysw"

description := "Companion buildable examples for 'The High Cost of AnyVal subclasses"

licenses in ThisBuild :=
  Seq("Apache License, Version 2.0"
	-> url("http://www.apache.org/licenses/LICENSE-2.0"))

homepage in ThisBuild := Some(url("http://www.example.com")) // TODO

scalaVersion in ThisBuild := "2.12.1"

crossScalaVersions in ThisBuild := Seq("2.12.1", "2.11.9", "2.10.6")

val scalazVersion = settingKey[String]("Version of scalaz to use.")

scalazVersion in ThisBuild := "7.2.10"

libraryDependencies ++= Seq(
    "org.scalaz" %% "scalaz-core" % scalazVersion.value
)

addCompilerPlugin("org.spire-math" %% "kind-projector" % "0.9.3" cross CrossVersion.binary)

SS.scalastyleFailOnError in ThisBuild := true

resolvers in ThisBuild += Opts.resolver.sonatypeReleases

publishMavenStyle in ThisBuild := true

publishTo in ThisBuild := {
  val ossrh = "https://oss.sonatype.org/"
  if (isSnapshot.value)
    Some("snapshots" at ossrh + "content/repositories/snapshots")
  else
    Some("releases"  at ossrh + "service/local/staging/deploy/maven2")
}

publishArtifact in Test := false

pomIncludeRepository in ThisBuild := { _ => false }

// TODO fix URLs below
pomExtra in ThisBuild := (
  <parent>
    <groupId>org.sonatype.oss</groupId>
    <artifactId>oss-parent</artifactId>
    <version>7</version>
  </parent>
  <scm>
    <url>https://code.launchpad.net/~scompall/sbt-adhoc-subprojects/trunk</url>
    <connection>scm:bazaar:http://bazaar.launchpad.net/~scompall/sbt-adhoc-subprojects/trunk/</connection>
  </scm>
  <developers>
    <developer>
      <id>S11001001</id>
      <name>Stephen Compall</name>
      <url>https://launchpad.net/~scompall</url>
    </developer>
  </developers>)
