package hcavsc.subst

object Labels {
  sealed abstract class LabelImpl {
    type T
    def subst[F[_]](fs: F[String]): F[T]
    def apply(s: String): T
    def unwrap(lbl: T): String
  }
  
  // do not forget `: LabelImpl`; it is key
  val Label: LabelImpl = new LabelImpl {
    type T = String
    override def subst[F[_]](fs: F[String]) = fs
    override def apply(s: String) = s
    override def unwrap(lbl: T) = lbl
  }

  type Label = Label.T

  import scalaz.Show

  implicit val showLabel: Show[Label] =
    Show shows {lbl =>
      s"Label(${Label.unwrap(lbl)})"}
}

import Labels._

class MyFirstTests {
  def printLabels(): Unit = {
    val fst = Label("hello")
    val snd = Label("world")
    identity(fst)
    (fst, snd)
    val lbls = List(fst, snd)
    lbls.map{x => Label(Label.unwrap(x) + "Aux")}
  }
}

object OtherTests {
  val untaggedList = List("hello", "world")
  val taggedList: List[Label] = Label.subst(untaggedList)

  val thenUntagged: List[String] =
    Label.subst[Lambda[x => List[x] => List[String]]](identity)(taggedList)

  val labelFunc: (Label, Int) => Label =
    Label.subst[Lambda[x => (x, Int) => x]](_ substring _)

  import scalaz.Monoid, scalaz.std.string._

  val labelMonoid: Monoid[Label] = Label.subst(Monoid[String])

  import scalaz.Leibniz, Leibniz.{===, refl}

  val typeEq: String === Label =
    Label.subst[String === ?](refl)

  val lme = labelMonoid.zero
  val lma = labelMonoid.append _
}
