package hcavsc.unidirectional

import scalaz.{Failure, Foldable, Success, ValidationNel}
import scalaz.syntax.std.option._
import scalaz.syntax.foldable._

object Labels {
  sealed abstract class LabelImpl {
    type T
    def widen[F[+_]](ft: F[T]): F[String]
    def narrow[F[_]: Foldable](fs: F[String])
        : ValidationNel[Err, F[T]]
    def apply(s: String): T
    def unwrap(lbl: T): String
  }

  type Err = Nothing // replace with desired error type

  // do not forget `: LabelImpl`; it is key
  val Label: LabelImpl = new LabelImpl {
    type T = String
    override def widen[F[+_]](ft: F[T]) = ft

    override def narrow[F[_]: Foldable](fs: F[String]) =
      fs.foldMap{string =>
        // return errors if not OK, INil() if OK
        scalaz.INil(): scalaz.IList[Err]
      }.toNel cata (Failure(_), Success(fs))

    override def apply(s: String) = s
    override def unwrap(lbl: T) = lbl
  }

  type Label = Label.T
}

import Labels._

object OtherTests {
  val untaggedList = List("hello", "world")
  val taggedList: List[Label] = untaggedList map Label.apply

  val thenUntagged: List[String] =
    Label.widen(taggedList)

  def report(xs: List[String]): Unit = ()

  def cwiden[F[-_]](fs: F[String]): F[Label] =
    Label.widen[Lambda[`+x` => F[x] => F[Label]]](identity)(fs)
 
  val lReport: List[Label] => Unit =
    cwiden[Lambda[`-x` => List[x] => Unit]](report)

  import scalaz.Liskov, Liskov.<~<

  val subtypeEv: Label <~< String =
    Label.widen[Lambda[`+x` => (Label <~< x)]](Liskov.refl)
}
